;****************************************************************
;
; MegaBlast - Spectravideo ver 1.00 (C) Electric Adventures 2014-2019
;
;****************************************************************
FNAME "MEGABLAST.ROM"
cpu z80
ORG 0000h

INCLUDE "SVIRom-Include.asm"

PATSIZE:   EQU 71
SPRITECOUNT: EQU 11

    ; disable interrupts
    di
    ; set stack pointer
    ld sp,0ffffh
    ; interrupt mode 1 i.e just RST38 generated
    im 1
    ; set screen mode immediately, as interrupts are enabled on entry
    call SETSCREEN2
    ; jump to application start
    jp INIT
    ; remaining is free space before the RST38
    DS 44

ORG 0038h
RST_38:
    jp NMI

; This is the start of your application
INIT:
    ; immediately initialise our Ram values
    CALL INITRAM

    ; initialise clock
    LD  HL,TIMER_TABLE
    LD  DE,TIMER_DATA_BLOCK
    CALL INIT_TIMER

    ;Enable timers - Tick, 1/4 sec and 1/2 sec timers
    CALL CREATE_TIMERS

    ;CALL SETINTERRUPT

    ; Seed random numbers with a fixed number (nothing else to use?)
    LD HL,1967
    CALL SEED_RANDOM


    ; Do all our VRAM setup
    ; NMI is currently disabled

TITLESCREEN:
    ; display our title screen
    DI
    ; Clear the screen
    CALL CLEARPAT


   	LD HL,OUTPUT_VDP_TITLE
	CALL SET_VDU_HOOK
    
    LD A,15
    LD (LEVEL),A
SPLASH_TITLE2:
	CALL JOYTST
	CP 1
	JR Z,NGAME
    LD	A,(TickTimer)
	CALL	TEST_SIGNAL
	OR	A
    JR Z,SPLASH_TITLE2
    ; Any other actions on the title screen go here
	JR SPLASH_TITLE2

NGAME:
    DI
    CALL INITRAM

MAIN_SCREEN:
    ; Read joysticks to clear any false reads
    CALL JOYTST

	LD HL,(TIME)
	LD (SEED),HL
	RR H
	RL L
	LD (SEED+2),HL

    ; disable interrupts
    DI

    LD A,1
    LD (LASTSCORE),A
    CALL DISPLAYSCORE

	LD HL,VDU_WRITES
	CALL SET_VDU_HOOK
    EI

    ; Main game logic loop
MLOOP:
    ; check that a base tick has occurred
    ; ensures consistent movement speed between 50 & 60Hz systems
	LD	A,(TickTimer)
	CALL	TEST_SIGNAL
	OR	A
	JR Z,MLOOP

    JR MLOOP

; Init Ram for a new game
INITRAM:
    LD A,3
    LD (LIVES),A
    LD HL,0
    LD (SCORE),HL
    LD (SCORE+1),HL
    LD A,1
    LD (LASTSCORE),A
    XOR A
    LD (LEVEL),A
    RET

; Display the current player lives (max 6)
DISPLAYLIVES:
    ; clear current lives display
    LD HL,VRAM_NAME+688
    LD BC,12
    XOR A
    CALL FILVRM
    LD HL,VRAM_NAME+720
    LD BC,12
    XOR A
    CALL FILVRM
    ; now show the current lives
    ; - first write the top characters
    LD D,67
    LD HL,VRAM_NAME+688
    CALL DL2
    ; - now write the bottom characters
    LD D,69
    LD HL,VRAM_NAME+720
DL2:
    CALL SETWRT
    LD A,(LIVES)
    LD B,A
DL1:
    LD A,D
    OUT (DATA_PORT),A
    INC A
    OUT (DATA_PORT),A
    DJNZ DL1
    RET

; Display the current score on the 2nd last row
DISPLAYSCORE:
    LD A,(SCORE)
    LD HL,LASTSCORE
    CP (HL)
    RET Z
    LD HL,VRAM_NAME + 707
    CALL SETWRT
    LD HL,SCORE+2
    LD B,3
SLP:
    CALL PRINTIT
    DEC HL
    DEC B
    LD A,B
    CP 0
    JR NZ,SLP
    LD A,(SCORE)
    LD (LASTSCORE),A
    RET

; This is our routine called every VDP interrupt during normal game play
; - Do all VDP writes here to avoid corruption
VDU_WRITES:
    CALL DISPLAYSCORE
    RET

; This is our routine called every VDP interrupt during the title screen
; - Do all VDP writes here to avoid corruption
OUTPUT_VDP_TITLE:
    RET


;**************************************************************************************************
; Include external data files
;**************************************************************************************************


;**************************************************************************************************
; Standard Libraries
;**************************************************************************************************

include "SVIRom-Lib.ASM"

END:	EQU $

;**************************************************************************************************
; RAM Definitions
;**************************************************************************************************

ORG RAMSTART

LEVEL:	     DS 1
LIVES:       DS 1
SCORE:	     DS 3
LASTSCORE:   DS 3



